# Copyright 2012-2018 Johannes Zarl-Zierl <johannes@zarl-zierl.at>
#
# This software is distributed WITHOUT ANY WARRANTY; without even the
# implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# This file is under the public domain and can be reused without restrictions.

if (NOT DEFINED BASE_DIR)
    message(FATAL_ERROR "UpdateVersion.cmake: BASE_DIR not set. "
                        "Please supply base working directory!")
endif()

# Version header locations
set(versionh_PATH ${BASE_DIR}/src/shared/version.h)
set(versionh_in_PATH ${BASE_DIR}/src/shared/version.h.in)
set(versionh_backup_PATH ${CMAKE_CURRENT_BINARY_DIR}/version.h~)

# git or tarball?
if (EXISTS ${BASE_DIR}/.git)
    # --> git:
    include(${CMAKE_CURRENT_LIST_DIR}/GitDescription.cmake)
    git_get_description(VERSION GIT_ARGS)
    if (NOT VERSION)
        set(VERSION "unknown")
    else()
        # Remove a trailing "v" from the version.
        string(REGEX REPLACE "^v" "" VERSION ${VERSION})
    endif()

    message(STATUS "Updating version information to ${VERSION}")
    # write version info to a temporary file
    configure_file(${versionh_in_PATH} ${versionh_backup_PATH})
    # update info if changed
    execute_process(COMMAND ${CMAKE_COMMAND} -E copy_if_different
                            ${versionh_backup_PATH} ${versionh_PATH})
    # make sure info doesn't get stale
    file(REMOVE ${versionh_backup_PATH})

else()
    # --> tarball
    if (NOT EXISTS ${versionh_PATH})
        message(SEND_ERROR "The generated file \"version.h\" does not exist!")
        message(AUTHOR_WARNING "When creating a release tarball, "
                               "please make sure to run cmake -P ${CMAKE_CURRENT_LIST_FILE}")
    endif()

endif()
