/* Copyright (c) 2011-2020 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes

#include "AboutDialog.h"

#include "shared/version.h"
#include "shared/SharedStyles.h"
#include "shared/SharedObjects.h"
#include "shared/Database.h"
#include "shared/ResourceFinder.h"
#include "shared/ServerProtocol.h"

// Qt includes
#include <QVBoxLayout>
#include <QGroupBox>
#include <QLabel>
#include <QDebug>
#include <QIcon>
#include <QDate>

AboutDialog::AboutDialog(QWidget *parent, SharedObjects *sharedObjects) : TitleDialog(parent)
{
    setWindowTitle(tr("Über Muckturnier"));
    addButtonBox(QDialogButtonBox::Ok);
    mainLayout()->setSizeConstraint(QLayout::SetFixedSize);

    const QDate compilerDate = QLocale(QStringLiteral("en_US")).toDate(
        QStringLiteral(__DATE__).simplified(), QStringLiteral("MMM d yyyy"));

    QVBoxLayout *headerLayout = new QVBoxLayout;
    headerLayout->setSpacing(0);
    layout()->addLayout(headerLayout);

    QLabel *banner = new QLabel;
    banner->setPixmap(QIcon(sharedObjects->resourceFinder()->find(
                                QStringLiteral("banner.svg"))).pixmap(QSize(320, 80)));
    banner->setAlignment(Qt::AlignCenter);
    banner->setStyleSheet(SharedStyles::STYLE_LOGO_HEADER);
    banner->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Maximum);
    headerLayout->addWidget(banner);

    QLabel *subtitle = new QLabel(tr("Das Programm für die Turnierleitung"));
    subtitle->setStyleSheet(SharedStyles::STYLE_LOGO_FOOTER);
    subtitle->setAlignment(Qt::AlignCenter);
    headerLayout->addWidget(subtitle);

    QGroupBox *versionBox = new QGroupBox;
    versionBox->setProperty("notitle", true);
    QVBoxLayout *versionBoxLayout = new QVBoxLayout(versionBox);
    layout()->addWidget(versionBox);

    QLabel *version = new QLabel(
          tr("<b>Version %1 (%2)</b><br/>").arg(
             QLatin1String(MT_VERSION), compilerDate.toString(QLatin1String("dd.MM.yyyy")))
        + tr("Datenbank: Revision %1<br/>").arg(sharedObjects->database()->dbVersion())
        + tr("Serverprotokoll: Revision %1<br/>").arg(ServerProtocol::PROTOCOL_VERSION));
    version->setTextInteractionFlags(Qt::TextBrowserInteraction | Qt::TextSelectableByMouse);
    version->setOpenExternalLinks(true);
    version->setAlignment(Qt::AlignCenter);
    versionBoxLayout->addWidget(version);

    QGroupBox *licenseBox = new QGroupBox;
    licenseBox->setProperty("notitle", true);
    QVBoxLayout *licenseBoxLayout = new QVBoxLayout(licenseBox);
    layout()->addWidget(licenseBox);

    QLabel *license = new QLabel(
        tr("Kompiliert gegen <a href=\"http://www.qt.io/\">Qt</a> %1<br/>").arg(
           QLatin1String(QT_VERSION_STR))
        + tr("Veröffentlicht unter der <a href=\"https://www.gnu.org/licenses/gpl-2.0.html\">"
             "GNU GPL v2</a>"));
    license->setAlignment(Qt::AlignCenter);
    license->setTextInteractionFlags(Qt::TextBrowserInteraction | Qt::TextSelectableByMouse);
    license->setOpenExternalLinks(true);
    licenseBoxLayout->addWidget(license);

    QGroupBox *authorBox = new QGroupBox;
    authorBox->setProperty("notitle", true);
    QVBoxLayout *authorBoxLayout = new QVBoxLayout(authorBox);
    layout()->addWidget(authorBox);

    QLabel *author = new QLabel(tr("Copyright %1 2011–%2 Tobias Leupold").arg(
                                   QStringLiteral("\u00A9"), QString::number(compilerDate.year()))
                                + tr("<br/>"
                                     "<a href=\"http://muckturnier.org/\">http://muckturnier.org/"
                                     "</a>"));
    author->setAlignment(Qt::AlignCenter);
    author->setTextInteractionFlags(Qt::TextBrowserInteraction | Qt::TextSelectableByMouse);
    author->setOpenExternalLinks(true);
    authorBoxLayout->addWidget(author);
}
