/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ASSIGNMENTPAGE_H
#define ASSIGNMENTPAGE_H

// Local includes
#include "shared/SharedObjects.h"
#include "shared/SearchEngine.h"

// Qt includes
#include <QWidget>

// Local classes
class TournamentSettings;
class SearchWidget;

// Qt classes
class QLabel;
class QTableWidget;

class AssignmentPage : public QWidget
{
    Q_OBJECT

public:
    explicit AssignmentPage(QWidget *parent, SharedObjects *sharedObjects);
    void updateAssignment(const QHash<int, QHash<int, QString>> &assignment,
                          const QHash<int, QHash<int, bool>> &pairComplete,
                          const QString &namesSeparator, int lastTable);

public slots:
    void resetSearch();

signals:
    void pageClosed();

private: // Enums
    enum EntryData {
        ActualText = Qt::UserRole
    };

private slots:
    void closePage();
    void updateSearch(const QString &searchText);

private: // Funnctions
    bool checkRowMatch(int row, SearchEngine::SearchType searchType) const;

private: // Variables
    TournamentSettings *m_tournamentSettings;
    SearchEngine *m_searchEngine;

    QLabel *m_noAssignment;
    QLabel *m_isSearch;
    QTableWidget *m_assignment;
    SearchWidget *m_searchWidget;

    QHash<int, bool> m_excludeFromSearch;

};

#endif // ASSIGNMENTPAGE_H
