/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CHECKSUMSDIALOG_H
#define CHECKSUMSDIALOG_H

// Local includes
#include "shared/TitleDialog.h"
#include "shared/TournamentSettings.h"
#include "shared/Database.h"

// Local classes
class SharedObjects;
class TournamentSettings;

// Qt classes
class QLabel;
class QPushButton;
class QTableWidget;
class QGroupBox;

class ChecksumsDialog : public TitleDialog
{
    Q_OBJECT

public:
    explicit ChecksumsDialog(QWidget *parent, SharedObjects *sharedObjects);

private slots:
    void showPartialChecksums();
    void divideFirstPart();
    void divideSecondPart();
    void resetPartialChecksums();

private: // Functions
    const QString checksum(const QString &text) const;
    const QString calculateChecksum(int startRow, int rowCount) const;
    void updateDividedChecksums();
    QTableWidget * createTableWidget();
    void updateRankingTable(QTableWidget *table, int startRow, int rowCount);

private: // Variables
    Database *m_db;
    TournamentSettings *m_tournamentSettings;

    QVector<Database::RankData> m_ranking;
    int m_firstStartRow;
    int m_secondStartRow;
    int m_firstRowCount;
    int m_secondRowCount;
    QPushButton *m_showPartialChecksums;
    QGroupBox *m_partialChecksumsBox;
    QGroupBox *m_firstPartBox;
    QLabel *m_firstPartChecksum;
    QGroupBox *m_secondPartBox;
    QLabel *m_secondPartChecksum;
    QPushButton *m_divideFirstPart;
    QPushButton *m_divideSecondPart;
    QPushButton *m_reset;
    QTableWidget *m_firstPartRanking;
    QTableWidget *m_secondPartRanking;

};

#endif // CHECKSUMSDIALOG_H
