/* Copyright (c) 2018-2020 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CLIENTPAGE_H
#define CLIENTPAGE_H

// Local includes
#include "shared/ServerProtocol.h"

// Qt includes
#include <QWidget>

// Local classes
class Database;
class PlayersPage;
class Client;
class LogDisplay;
class SocketAddressWidget;
class SharedObjects;
class StringTools;
class Database;
class DiscoverClient;

// Qt classes
class QPushButton;
class QLabel;
class QGroupBox;
class QTableWidget;

class ClientPage : public QWidget
{
    Q_OBJECT

public:
    explicit ClientPage(QWidget *parent, SharedObjects *sharedObjects, PlayersPage *playersPage,
                        const QString &ip, int port);
    Client * client() const;
    void closePage();

public slots:
    void showDifferingNames(QStringList &localNames, QStringList &remoteNames);

signals:
    void connectionRequested(const QString &ip, int port);
    void connected(Client *client);
    void connectionClosed();
    void statusUpdate(const QString &text);
    void pageClosed();
    void nameEdited();
    void namesDifferencesDisplayed(bool state);
    void raiseMe();

private slots:
    void finishConstruction(bool ipGiven);
    void connectToServer();
    void closeConnection();
    void checkConnectResult(bool success, const QString &error);
    void connectionError(const QString &text);
    void requestRejected(ServerProtocol::RejectReason reason);
    void adoptList();
    void cancelAdoption();
    void searchServer();
    void processDiscoverResult(const QString &ip, int port);

private: // Functions
    void resetInterface();
    bool checkConnected();
    void closeClient();

private: // Variables
    Client *m_client;
    QGroupBox *m_settingsBox;
    SocketAddressWidget *m_socketAddress;
    QLabel *m_warningLabel;
    QPushButton *m_searchServerButton;
    QPushButton *m_connectButton;
    QPushButton *m_disconnectButton;
    QPushButton *m_closeButton;
    QPushButton *m_adoptListButton;
    QPushButton *m_cancelButton;
    QGroupBox *m_displayBox;
    LogDisplay *m_display;
    QLabel *m_connectedLabel;
    bool m_connecting;
    QGroupBox *m_differencesBox;
    QTableWidget *m_differences;
    StringTools *m_stringTools;
    Database *m_db;
    QString m_notConnectedMessage;
    DiscoverClient *m_discoverClient;
    int m_discoverAttempts;
    int m_currentDiscoverAttempts = 0;

};

#endif // CLIENTPAGE_H
