/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ASSIGNMENTPOPUP_H
#define ASSIGNMENTPOPUP_H

// Local includes
#include "shared/SelectionPopup.h"

// Qt classes
class QSpinBox;
class QRadioButton;
class QLabel;
class QPushButton;
class QCheckBox;

class AssignmentPopup : public SelectionPopup
{
    Q_OBJECT

public:
    explicit AssignmentPopup(QWidget *parent);
    void show(bool existing, bool markerSet, bool isSinglePlayer, int table, int pair,
              bool restore = false, const QPoint &position = QPoint(-1, -1));
    void setAssignedTables(QVector<int> &assignedPair1Tables, QVector<int> &assignedPair2Tables);
    int table() const;
    int pair() const;
    bool removeMarker() const;
    void setPairMode(bool state);

private slots:
    void checkSelection();

private: // Functions
    int findFirstFree(const QVector<int> &list) const;

private: // Variables
    QVector<int> m_assignedPair1Tables;
    QVector<int> m_assignedPair2Tables;
    QWidget *m_main;
    QWidget *m_selection;
    QLabel *m_isSinglePlayer;
    QSpinBox *m_table;
    QRadioButton *m_pair1;
    QRadioButton *m_pair2;
    QLabel *m_taken;
    QPushButton *m_save;
    QCheckBox *m_removeMarker;
    QPushButton *m_delete;
    bool m_markerSet;
    int m_assignmentsPerPair;

};

#endif // ASSIGNMENTPOPUP_H
