/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes
#include "MarkersPopup.h"

// Qt includes
#include <QDebug>
#include <QVBoxLayout>
#include <QPushButton>

// C++ includes
#include <functional>

MarkersPopup::MarkersPopup(QWidget *parent) : SelectionPopup(parent)
{
}

void MarkersPopup::show(const int currentMarker, const QPoint &position)
{
    for (QPushButton *button : findChildren<QPushButton *>()) {
        button->setVisible(button->property("id").toInt() != currentMarker);
    }

    if (position == QPoint(-1, -1)) {
        SelectionPopup::show();
    } else {
        SelectionPopup::show(position);
    }
}

void MarkersPopup::updateMarkers(const QVector<Muckturnier::Marker> &markers)
{
    QWidget *widget = new QWidget;
    QVBoxLayout *layout = new QVBoxLayout(widget);
    layout->setSpacing(0);

    for (const Muckturnier::Marker &marker : markers) {
        QPushButton *button = new QPushButton(marker.name);
        button->setStyleSheet(QStringLiteral("color:") + marker.color.name());
        button->setProperty("id", QVariant(marker.id));
        layout->addWidget(button);
        connect(button, &QPushButton::clicked,
                std::bind(&MarkersPopup::markerSelected, this, button));
    }

    setScrollWidget(widget);
}

void MarkersPopup::markerSelected(QPushButton *button)
{
    m_markerId = button->property("id").toInt();
    m_markerName = m_markerId != 0 ? button->text() : QString();
    accept();
}

QString MarkersPopup::markerName() const
{
    return m_markerName;
}

int MarkersPopup::markerId() const
{
    return m_markerId;
}
