/* Copyright (c) 2011-2020 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes
#include "BoogerSpinBox.h"

// Qt includes
#include <QDebug>
#include <QLineEdit>
#include <QFocusEvent>
#include <QTimer>

BoogerSpinBox::BoogerSpinBox(QWidget *parent) : QSpinBox(parent)
{
    setWrapping(true);
    connect(lineEdit(), &QLineEdit::returnPressed, this, &BoogerSpinBox::returnPressed);
}

void BoogerSpinBox::contextMenuEvent(QContextMenuEvent *)
{
    emit scorePopupRequested(this);
}

void BoogerSpinBox::focusInEvent(QFocusEvent *event)
{
    QSpinBox::focusInEvent(event);
    if (event->reason() == Qt::MouseFocusReason || event->reason() == Qt::TabFocusReason) {
        QTimer::singleShot(0, this, &QSpinBox::selectAll);
    }
}

void BoogerSpinBox::deselect()
{
    lineEdit()->deselect();
}
