/* Copyright (c) 2011-2020 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SCOREPAGE_H
#define SCOREPAGE_H

// Local includes
#include "shared/Database.h"

// Qt includes
#include <QWidget>

// Local classes
class SharedObjects;
class StringTools;
class TournamentSettings;
class SearchEngine;
class BoogerWidget;
class TitleMenu;
class NumbersComboBox;
class PlayersComboBox;
class BoogerSpinBox;
class ScorePopup;
class SearchWidget;
class Settings;
class TableComboBox;

// Qt classes
class QGroupBox;
class QCheckBox;
class QScrollArea;
class QPushButton;
class QTableWidget;
class QAction;
class QLabel;

class ScorePage : public QWidget
{
    Q_OBJECT

public:
    explicit ScorePage(QWidget *parent, SharedObjects *sharedObjects);
    void setupGui();
    void reload(int lastRound = -1);
    void setScrollAreaHeight();
    void setNoTableNumbers(bool state);
    void setAutoSelectPairs(bool state);
    void playersNameEdited();

public slots:
    void resetSearch();
    void updateAssignment(int round);

signals:
    void statusUpdate(const QString &message);
    void tournamentStarted();
    void scoreChanged();

protected:
    virtual void showEvent(QShowEvent *event) override;

private slots:
    void openRound(int round);
    void tableChanged(int table);
    void saveScore();
    void showContextMenu(QPoint point);
    void editScore();
    void deleteScore();
    void checkEnterScore();
    void updateSearch(const QString &searchText);
    void blockSelectSignals(bool state);
    void swapPairs();
    void noTableNumbersChanged(bool state);
    void autoSelectPairsChanged(bool state);
    void boogerAbortedStateChanged(int booger, bool aborted);
    void checkSavePossible();
    void showScorePopup(BoogerSpinBox *spinBox);
    void setSelectedScore();

private: // Functions
    void updateScore();
    void playersChanged(int pair, int pairId);
    void doDeleteScore(int round, int table);
    QString getStatusUpdateIdentifier() const;
    QString formatOpponentsString(const QString &pair1, const QString &pair2) const;
    void fillInScore(int round, int table, const QVector<Database::BoogerResult> &score);
    QVector<Database::BoogerResult> currentScore() const;

private: // Variables
    Database *m_db;
    Settings *m_settings;
    TournamentSettings *m_tournamentSettings;
    StringTools *m_stringTools;
    SearchEngine *m_searchEngine;
    ScorePopup *m_scorePopup;
    BoogerSpinBox *m_targetSpinBox;

    NumbersComboBox *m_roundSelect;
    QCheckBox *m_noTableNumbers;

    QGroupBox *m_resultBox;
    TableComboBox *m_tableSelect;
    QHash<int, QHash<int, PlayersComboBox *>> m_playersSelect;
    QCheckBox *m_autoSelectPairs;
    QScrollArea *m_scoreBoxScroll;
    QVector<BoogerWidget *> m_boogerWidgets;
    QPushButton *m_saveScoreButton;

    QGroupBox *m_roundSummaryBox;
    QTableWidget *m_scoreList;

    SearchWidget *m_searchWidget;
    QLabel *m_isSearchLabel;

    TitleMenu *m_contextMenu;
    QAction *m_editAction;
    QAction *m_deleteAction;
    QAction *m_swapPairsAction;
    QLabel *m_tableLabel;

    bool m_lastAutoSelectState;
    int m_lastOpenedRound;
    int m_selectedTable;
    Database::Assignment m_assignment;
    int m_scoreSize;

};

#endif // SCOREPAGE_H
