/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SERVERPAGE_H
#define SERVERPAGE_H

// Qt includes
#include <QWidget>

// Local classes
class SharedObjects;
class PlayersPage;
class Server;
class SocketAddressWidget;
class LogDisplay;
class DiscoverServer;

// Qt classes
class QLabel;
class QPushButton;
class QGroupBox;

class ServerPage : public QWidget
{
    Q_OBJECT

public:
    explicit ServerPage(QWidget *parent, SharedObjects *sharedObjects, PlayersPage *playersPage);
    bool checkServerRunning();
    void closePage();

signals:
    void serverStarted(Server *server);
    void serverStopped();
    void statusUpdate(const QString &text);
    void pageClosed();

private slots:
    void startServer();
    void stopServer();
    void updateClientCount(int count);

private: // Functions
    void closeServer();
    bool checkConnections(const QString &messageTitle = QString());

private: // Variables
    Server *m_server;
    SocketAddressWidget *m_socketAddress;
    QGroupBox *m_settingsBox;
    QPushButton *m_startServer;
    QPushButton *m_stopServer;
    QPushButton *m_closeButton;
    QLabel *m_address;
    QGroupBox *m_displayBox;
    LogDisplay *m_display;
    QString m_serverStoppedMessage;
    DiscoverServer *m_discoverServer;

};

#endif // SERVERPAGE_H
