/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes

#include "FileNameTemplatePage.h"

#include "shared/Settings.h"
#include "shared/defaultValues.h"

// Qt includes
#include <QDebug>
#include <QVBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QDateTime>
#include <QMessageBox>

FileNameTemplatePage::FileNameTemplatePage(QWidget *parent, Settings *settings)
    : AbstractSettingsPage(parent, tr("Vorlage Datenbankdateiname")),
      m_settings(settings)
{
    QLabel *fileNameTemplateLabel = new QLabel(tr("Für neue Turnierdatenbanken wird der Dateiname "
                                                  "aufgrund folgender Vorlage voreingestellt:"));
    fileNameTemplateLabel->setWordWrap(true);
    layout()->addWidget(fileNameTemplateLabel);

    m_template = new QLineEdit;
    connect(m_template, &QLineEdit::textChanged, this, &FileNameTemplatePage::updateParsedFileName);
    layout()->addWidget(m_template);

    m_parsedFileName = new QLabel;
    layout()->addWidget(m_parsedFileName);

    m_template->setText(m_settings->dbFileNameTemplate());

    QLabel *formatCommentLabel = new QLabel(tr("Es können Teile des aktuellen Datums und/oder der "
                                               "aktuellen Zeit eingefügt werden. "
                                               "<a href=\"#info\">Wie funktioniert das?</a>"));
    formatCommentLabel->setWordWrap(true);
    formatCommentLabel->setTextInteractionFlags(Qt::TextBrowserInteraction);
    layout()->addWidget(formatCommentLabel);
    connect(formatCommentLabel, &QLabel::linkActivated,
            this, &FileNameTemplatePage::showFileNameTemplateInfo);

    layout()->addStretch();

    QHBoxLayout *controlLayout = new QHBoxLayout;
    layout()->addLayout(controlLayout);
    controlLayout->addStretch();
    QPushButton *restoreTemplateButton = new QPushButton(tr("Zurücksetzen"));
    connect(restoreTemplateButton, &QPushButton::clicked,
            this, &FileNameTemplatePage::restoreTemplate);
    controlLayout->addWidget(restoreTemplateButton);
}

void FileNameTemplatePage::updateParsedFileName(const QString &text)
{
    m_parsedFileName->setText(tr("Geparste Vorlage: „%1.mtdb“").arg(
                                 QDateTime::currentDateTime().toString(text.simplified())));
}

void FileNameTemplatePage::showFileNameTemplateInfo(const QString &)
{
    QMessageBox::information(this, tr("Vorlage für Datenbankdateinamen"),
        tr("<p>Zum Parsen (Übersetzen/Umsetzen) der Vorlage wird die Funktion "
           "<kbd>QDateTime::toString()</kbd> benutzt. Die genaue Syntax ist in der "
           "<a href=\"https://doc.qt.io/qt-5/qdatetime.html#toString\">Dokumentation zu "
           "<kbd>QDateTime</kbd></a> beschrieben.</p>"
           "<p>Für diesen Einsatzzweck am relevantesten:</p>"
           "<ul>"
           "<li>Alles in einfachen Anführungszeichen <kbd>'</kbd> wird genau so angezeigt</li>"
           "<li><kbd>yy</kbd> bzw. <kbd>yyyy</kbd> wird mit der aktuellen Jahreszahl ersetzt</li>"
           "<li><kbd>M</kbd> bis <kbd>MMMM</kbd> wird mit dem aktuellen Monat ersetzt</li>"
           "</ul>"));
}

void FileNameTemplatePage::restoreTemplate()
{
    m_template->setText(QStringLiteral(DEFAULT_DB_FILE_NAME_TEMPLATE));
}

void FileNameTemplatePage::saveSettings()
{
    const QString templateText = m_template->text().simplified();
    m_settings->saveDbFileNameTemplate(templateText.isEmpty()
        ? QStringLiteral(DEFAULT_DB_FILE_NAME_TEMPLATE) : templateText);
}
