/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MARKERSTEMPLATEPAGE_H
#define MARKERSTEMPLATEPAGE_H

// Local includes

#include "AbstractSettingsPage.h"

#include "shared/Muckturnier.h"

// Local classes
class SharedObjects;
class Database;
class Settings;
class TournamentSettings;

// Qt classes
class QTableWidget;
class QGroupBox;
class QLabel;

class MarkersTemplatePage : public AbstractSettingsPage
{
    Q_OBJECT

public:
    explicit MarkersTemplatePage(QWidget *parent, SharedObjects *sharedObjects);
    virtual void saveSettings() override;

private slots:
    void restoreTemplate();
    void adoptCurrentSettings();

private: // Functions
    void updateTemplateDisplay();
    QString markerName(int id) const;

private: // Variables
    Database *m_db;
    Settings *m_settings;
    TournamentSettings *m_tournamentSettings;

    QVector<Muckturnier::Marker> m_markersTemplate;
    QTableWidget *m_markersTemplateDisplay;
    Muckturnier::SinglesManagement m_singlesManagementTemplate;
    QLabel *m_noProcessSingles;
    QGroupBox *m_processSingles;
    QLabel *m_singlesMarker;
    QLabel *m_automaticMarking;
    QLabel *m_assignedMarker;
    int m_defaultMarker;
    QLabel *m_defaultMarkerLabel;
    int m_absentMarker;
    QLabel *m_absentMarkerLabel;

};

#endif // MARKERSTEMPLATEPAGE_H
