/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes

#include "PlayersPagePage.h"

#include "shared/Settings.h"

// Qt includes
#include <QDebug>
#include <QVBoxLayout>
#include <QLabel>
#include <QCheckBox>
#include <QHBoxLayout>
#include <QPushButton>

PlayersPagePage::PlayersPagePage(QWidget *parent, Settings *settings)
    : AbstractSettingsPage(parent, tr("Anmeldeseite")),
      m_settings(settings)
{
    QLabel *header = new QLabel(tr("Neben der „Name“-Spalte auf der Anmeldeseite (die immer "
                                   "sichtbar ist) werden standardmäßig folgende Spalten "
                                   "eingeblendet:"));
    header->setWordWrap(true);
    layout()->addWidget(header);

    const QHash<PlayersPageColumns::Column, bool> visible = m_settings->playersPageColumns();

    for (Column column : ALL_COLUMNS) {
        m_columnVisible[column] = new QCheckBox(tr(CHECKBOX_TEXT.value(column)));
        m_columnVisible[column]->setChecked(visible.value(column));
        layout()->addWidget(m_columnVisible[column]);
    }

    layout()->addStretch();

    QHBoxLayout *resetLayout = new QHBoxLayout;
    layout()->addLayout(resetLayout);
    resetLayout->addStretch();
    QPushButton *resetButton = new QPushButton(tr("Zurücksetzen"));
    connect(resetButton, &QPushButton::clicked, this, &PlayersPagePage::restoreDefaultValues);
    resetLayout->addWidget(resetButton);

}

void PlayersPagePage::restoreDefaultValues()
{
    for (Column column : ALL_COLUMNS) {
        m_columnVisible[column]->setChecked(DEFAULT_VISIBILITY.value(column));
    }
}

void PlayersPagePage::saveSettings()
{
    QHash<PlayersPageColumns::Column, bool> visibility;
    for (Column column : ALL_COLUMNS) {
        visibility.insert(column, m_columnVisible[column]->isChecked());
    }
    m_settings->savePlayersPageColumns(visibility);
}
