/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes

#include "SaveOnClosePage.h"

#include "shared/Settings.h"

// Qt includes
#include <QDebug>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QCheckBox>
#include <QPushButton>
#include <QMessageBox>

SaveOnClosePage::SaveOnClosePage(QWidget *parent, Settings *settings)
    : AbstractSettingsPage(parent, tr("Beim Beenden speichern")),
      m_settings(settings)
{
    QLabel *saveOnCloseLabel = new QLabel(tr("Beim Beenden des Programms können folgende "
                                             "Einstellungen gespeichert und beim nächsten Start "
                                             "wiederhergestellt werden:"));
    saveOnCloseLabel->setWordWrap(true);
    layout()->addWidget(saveOnCloseLabel);

    m_saveWindowGeometry = new QCheckBox(tr("Position und Größe des Programmfensters"));
    m_saveWindowGeometry->setChecked(m_settings->saveMainWindowGeometryEnabled());
    layout()->addWidget(m_saveWindowGeometry);

    m_saveLastDbPath = new QCheckBox(tr("Verzeichnis der letzten benutzen Turnierdatenbank\n"
                                        "(als Startpunkt des „Turnier öffnen“-Dateidialogs)"));
    m_saveLastDbPath->setChecked(m_settings->saveLastDbPathEnabled());
    layout()->addWidget(m_saveLastDbPath);

    layout()->addStretch();

    QHBoxLayout *resetLayout = new QHBoxLayout;
    layout()->addLayout(resetLayout);
    resetLayout->addStretch();
    QPushButton *resetButton = new QPushButton(tr("Zurücksetzen"));
    connect(resetButton, &QPushButton::clicked, this, &SaveOnClosePage::restoreDefaultValues);
    resetLayout->addWidget(resetButton);
}

void SaveOnClosePage::saveSettings()
{
    m_settings->enableSaveMainWindowGeometry(m_saveWindowGeometry->isChecked());
    m_settings->enableSaveLastDbPath(m_saveLastDbPath->isChecked());
}

void SaveOnClosePage::restoreDefaultValues()
{
    m_saveWindowGeometry->setChecked(true);
    m_saveLastDbPath->setChecked(true);
}
