/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes

#include "SettingsDialog.h"
#include "AbstractSettingsPage.h"
#include "FileNameTemplatePage.h"
#include "TournamentTemplatePage.h"
#include "MarkersTemplatePage.h"
#include "StringsPage.h"
#include "SaveOnClosePage.h"
#include "NetworkPage.h"
#include "PlayersPagePage.h"

#include "shared/SharedObjects.h"

// Qt includes
#include <QDebug>
#include <QVBoxLayout>
#include <QListWidget>
#include <QStackedWidget>

SettingsDialog::SettingsDialog(QWidget *parent, SharedObjects *sharedObjects)
    : TitleDialog(parent)
{
    setTitle(tr("Einstellungen"));
    addButtonBox(QDialogButtonBox::Save | QDialogButtonBox::Close);
    buttonBox()->button(QDialogButtonBox::Save)->setText(tr("Speichern und schließen"));

    QHBoxLayout *mainLayout = new QHBoxLayout;
    layout()->addLayout(mainLayout);

    m_pagesSelection = new QListWidget;
    connect(m_pagesSelection, &QListWidget::currentItemChanged, this, &SettingsDialog::changePage);
    mainLayout->addWidget(m_pagesSelection);

    m_pages = new QStackedWidget;
    mainLayout->addWidget(m_pages);

    const QVector<AbstractSettingsPage *> pages {
        new FileNameTemplatePage(this, sharedObjects->settings()),
        new TournamentTemplatePage(this, sharedObjects),
        new MarkersTemplatePage(this, sharedObjects),
        new PlayersPagePage(this, sharedObjects->settings()),
        new StringsPage(this, sharedObjects->settings()),
        new SaveOnClosePage(this, sharedObjects->settings()),
        new NetworkPage(this, sharedObjects->settings())
    };

    for (AbstractSettingsPage *page : pages) {
        m_pagesSelection->addItem(page->pageTitle());
        m_pages->addWidget(page);
    }

    const int pagesSelectionWidth = m_pagesSelection->sizeHintForColumn(0) + 4;
    m_pagesSelection->setMinimumWidth(pagesSelectionWidth);
    m_pagesSelection->setMaximumWidth(pagesSelectionWidth);
    m_pagesSelection->setCurrentRow(0);
    m_pagesSelection->setFocus();
}

void SettingsDialog::changePage(QListWidgetItem *item, QListWidgetItem *)
{
    m_pages->setCurrentIndex(m_pagesSelection->row(item));
}

void SettingsDialog::accept()
{
    for (int i = 0; i < m_pages->count(); i++) {
        qobject_cast<AbstractSettingsPage *>(m_pages->widget(i))->saveSettings();
    }
    TitleDialog::accept();
}
