/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef ABSTRACTDISCOVERENGINE_H
#define ABSTRACTDISCOVERENGINE_H

// Local includes
#include "AbstractJsonEngine.h"

// Qt includes
#include <QHostAddress>

// Qt classes
class QUdpSocket;
class QJsonObject;

class AbstractDiscoverEngine : public AbstractJsonEngine
{
    Q_OBJECT

public:
    explicit AbstractDiscoverEngine(QObject *parent, int port);
    bool listen();
    QString errorString();
    void close();
    int port() const;

signals:
    void invalidData(const QHostAddress &address);

protected: // Functions
    virtual void processData(const QJsonObject &jsonObject, const QHostAddress &senderAddress) = 0;
    void broadcastData(const QJsonObject &jsonObject);
    void sendData(const QJsonObject &jsonObject, const QHostAddress &address);

private slots:
    void readDatagram();

private: // Variables
    QUdpSocket *m_socket;
    const int m_port;

};

#endif // ABSTRACTDISCOVERENGINE_H
