/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

// Local includes
#include "AbstractJsonEngine.h"

// Qt includes
#include <QJsonDocument>
#include <QJsonObject>

AbstractJsonEngine::AbstractJsonEngine(QObject *parent) : QObject(parent)
{
}

QByteArray AbstractJsonEngine::jsonObjectToByteArray(const QJsonObject &jsonObject) const
{
    return QJsonDocument(jsonObject).toJson(QJsonDocument::Compact);
}

QJsonObject AbstractJsonEngine::jsonObjectFromByteArray(const QByteArray &byteArray,
                                                        bool *success) const
{
    QJsonParseError error;
    const QJsonDocument json = QJsonDocument::fromJson(byteArray, &error);
    if (error.error != QJsonParseError::NoError || ! json.isObject()) {
        (*success) = false;
        return QJsonObject();
    }

    (*success) = true;
    return json.object();
}
