/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MUCKTURNIER_H
#define MUCKTURNIER_H

#include <QString>
#include <QHash>
#include <QColor>

namespace Muckturnier
{

// Tournament Mode

enum TournamentMode {
    FixedPairs,
    SinglePlayers
};

const QString TOURNAMENT_MODE_FIXED_PAIRS    = QStringLiteral("fixed pairs");
const QString TOURNAMENT_MODE_SINGLE_PLAYERS = QStringLiteral("single players");
const QHash<TournamentMode, QString> TOURNAMENT_MODE_MAP {
    { TournamentMode::FixedPairs,    TOURNAMENT_MODE_FIXED_PAIRS },
    { TournamentMode::SinglePlayers, TOURNAMENT_MODE_SINGLE_PLAYERS }
};

// Markers

enum MarkerSorting {
    NewBlock,
    Merged
};

struct Marker {
    int id;
    QString name;
    QColor color;
    MarkerSorting sorting;
};

// Singles Management

constexpr int SINGLES_MANAGEMENT_VERSION = 1;

struct SinglesManagement {
    bool enabled;
    int singleMarker;
    bool markSingles;
    QString conditionString;
    int condition;
    int assignedMarker;
};

const SinglesManagement DEFAULT_SINGLES_MANAGEMENT {
    false,               // enabled
    0,                   // singleMarker
    false,               // markSingles
    QStringLiteral("/"), // conditionString
    0,                   // condition
    0                    // assignedMarker
};

}

#endif // MUCKTURNIER_H
