/* Copyright (c) 2018-2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SEARCHENGINE_H
#define SEARCHENGINE_H

// Qt includes
#include <QObject>
#include <QHash>
#include <QVector>

// Local classes
class StringTools;

class SearchEngine : public QObject
{
    Q_OBJECT

public:
    enum SearchType {
        Default,
        DrawnPairsScore,
        DrawnPairsPreAssignment
    };

    explicit SearchEngine(QObject *parent, StringTools *stringTools);
    bool checkMatch(const QString &text, SearchType searchType = SearchType::Default) const;
    void clearSearchCache(SearchType searchType = SearchType::Default);
    void updateSearchCache(const QString &name, SearchType searchType = SearchType::Default);
    void setSearchTerm(const QString &text);
    QHash<QString, QStringList> checkListForPossibleDuplicates(const QStringList &list);

signals:
    void progress(int percent);

private: // Functions
    QVector<QString> searchVariants(const QString &text);
    void addAllVariants(const QString &searchTerm, QVector<QString> &variantsList);
    QString replaceUmlauts(QString text) const;
    QString stripDiacritis(const QString &text) const;
    bool checkMatch(const QString &text,
                    const QHash<QString, QVector<QString>> &variantsList) const;

private: // Variables
    StringTools *m_stringTools;
    const QHash<SearchType, QHash<QString, QVector<QString>> *> m_variantsCache;
    QHash<QString, QVector<QString>> m_defaultVariants;
    QHash<QString, QVector<QString>> m_drawnPairsScoreVariants;
    QHash<QString, QVector<QString>> m_drawnPairsPreAssignmentVariants;
    QVector<QString> m_searchTermRegularExpressions;

};

#endif // SEARCHENGINE_H
