/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef SETTINGS_H
#define SETTINGS_H

// Local includes
#include "PlayersPageColumns.h"
#include "Muckturnier.h"

// Qt includes
#include <QString>
#include <QSettings>

// Qt classes
class QAction;

class Settings : public QSettings
{
    Q_OBJECT

public:
    struct TournamentTemplate {
        Muckturnier::TournamentMode tournamentMode;
        int boogerScore;
        int boogersPerRound;
        bool autoSelectPairs;
        bool noTableNumbers;
    };

    explicit Settings(QObject *parent);

    bool saveMainWindowGeometryEnabled() const;
    void enableSaveMainWindowGeometry(bool state);
    void saveMainWindowGeometry(const QByteArray &data);
    QByteArray mainWindowGeometry() const;

    bool saveLastDbPathEnabled() const;
    void enableSaveLastDbPath(bool state);
    void saveLastDbPath(const QString &dbPath);
    QString lastDbPath() const;

    void saveTournamentTemplate(const TournamentTemplate &tournamentTemplate);
    TournamentTemplate defaultTournamentTemplate() const;
    TournamentTemplate tournamentTemplate() const;

    void saveDbFileNameTemplate(const QString &text);
    QString dbFileNameTemplate() const;

    void saveMarkersTemplate(const QVector<Muckturnier::Marker> &markers);
    QVector<Muckturnier::Marker> defaultMarkersTemplate() const;
    QVector<Muckturnier::Marker> markersTemplate() const;

    void saveDefaultMarker(int id);
    int defaultMarker() const;

    void saveAbsentMarker(int id);
    int absentMarker() const;

    void saveSinglesManagementTemplate(const Muckturnier::SinglesManagement &data);
    Muckturnier::SinglesManagement defaultSinglesManagementTemplate();
    Muckturnier::SinglesManagement singlesManagementTemplate();

    void saveBoogerSymbol(const QString &text);
    QString boogerSymbol() const;
    void saveNamesSeparator(const QString &text);
    QString namesSeparator() const;

    void saveServerPort(int port);
    int serverPort() const;
    void saveConnectTimeout(int timeout);
    int connectTimeout() const;
    void saveRequestTimeout(int timeout);
    int requestTimeout() const;
    void saveDiscoverPort(int port);

    int discoverPort() const;
    void saveDiscoverTimeout(int timeout);
    int discoverTimeout() const;
    void saveDiscoverAttempts(int attempts);
    int discoverAttempts() const;

    void savePreferIpv6(bool state);
    bool preferIpv6() const;

    void savePlayersPageColumns(const QHash<PlayersPageColumns::Column, bool> &visible);
    QHash<PlayersPageColumns::Column, bool> playersPageColumns() const;

private: // Functions
    bool parseMarkerData(int &id, const QJsonValue &value,
                         QVector<Muckturnier::Marker> &markers) const;
    void appendMarker(const QString &name, const QColor &color, Muckturnier::MarkerSorting sorting,
                      int id, QVector<Muckturnier::Marker> &markers) const;

};

#endif // SETTINGS_H
