/* Copyright (c) 2019 Tobias Leupold <tobias.leupold@gmx.de>

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation in version 2 of the License.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef TOURNAMENTSETTINGS_H
#define TOURNAMENTSETTINGS_H

// Local includes
#include "Muckturnier.h"
#include "PlayersPageColumns.h"

// Qt includes
#include <QObject>

// Local classes
class Settings;

class TournamentSettings : public QObject
{
    Q_OBJECT

public:
    explicit TournamentSettings(QObject *parent, Settings *settings);
    void initialize();
    bool load(const QString &data);
    QString toString() const;

    void setMuckturnierVersion(const QString &muckturnierVersion);
    const QString &muckturnierVersion() const;

    void setTournamentMode(Muckturnier::TournamentMode tournamentMode);
    Muckturnier::TournamentMode tournamentMode() const;
    bool isPairMode() const;
    bool isSinglePlayerMode() const;

    void setBoogerScore(int score);
    int boogerScore() const;

    void setBoogersPerRound(int count);
    int boogersPerRound() const;

    void setNoTableNumbers(bool state);
    bool noTableNumbers() const;

    void setAutoSelectPairs(bool state);
    bool autoSelectPairs() const;

    void setIncludeOpponentGoals(bool state);
    bool includeOpponentGoals() const;

    void setDefaultMarker(int id);
    int defaultMarker() const;

    void setAbsentMarker(int id);
    int absentMarker() const;

    void setSinglesManagement(const Muckturnier::SinglesManagement &data);
    const Muckturnier::SinglesManagement &singlesManagement() const;

    void setPlayersPageColumns(const QHash<PlayersPageColumns::Column, bool> &data);
    const QHash<PlayersPageColumns::Column, bool> &playersPageColumns() const;

private: // Variables
    Settings *m_settings;

    QString m_muckturnierVersion;
    Muckturnier::TournamentMode m_tournamentMode;
    int m_boogerScore;
    int m_boogersPerRound;
    bool m_noTableNumbers;
    bool m_autoSelectPairs;
    bool m_includeOpponentGoals;
    int m_defaultMarker;
    int m_absentMarker;
    Muckturnier::SinglesManagement m_singlesManagement;
    QHash<PlayersPageColumns::Column, bool> m_playersPageColumns;

};

#endif // TOURNAMENTSETTINGS_H
